"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var matcher_1 = require("matcher");
var trackers_1 = __importDefault(require("./trackers"));
var DiscordTracker = /** @class */ (function () {
    /**
     * @param client The discord client that we will work with (use v11 to retain the ability to use custom tokens)
     */
    function DiscordTracker(client) {
        /**
         * Whether to catch discord updates or not
         */
        this.working = false;
        /**
         * The list of user IDs that we are tracking
         */
        this.users = new Set();
        /**
         * The list of handlers that we are tracking
         */
        this.handlers = new Map();
        this.client = client;
        trackers_1.default.forEach(this.registerTracker.bind(this));
    }
    DiscordTracker.prototype.registerTracker = function (Tracker) {
        var tracker = new Tracker(this.send.bind(this));
        this.client.on(tracker.updateType, tracker.handler.bind(tracker));
    };
    /**
     * Used to dispatch events of type CommonEvent
     * @param event The CommonEvent object
     */
    DiscordTracker.prototype.send = function (event) {
        if (!this.working)
            return;
        if (!this.users.has(event.author.id))
            return;
        for (var _i = 0, _a = Array.from(this.handlers); _i < _a.length; _i++) {
            var _b = _a[_i], wildcard = _b[0], handler = _b[1];
            if (!matcher_1.isMatch(event.type, wildcard))
                continue;
            handler(event);
        }
    };
    /**
     * Start the Discord stalker
     */
    DiscordTracker.prototype.start = function () {
        this.working = true;
    };
    /**
     * Stop the discord stalker
     */
    DiscordTracker.prototype.stop = function () {
        this.working = false;
    };
    return DiscordTracker;
}());
exports.default = DiscordTracker;
